import sys
from utils import getDASStrainRate
from lines_number import line_ch_dict
from readSEPfile import extract_coords_from_channel_file, reorder_data
import argparse
from obspy import UTCDateTime
import os
from plot_SDASA_data import plot_SDASA_data
import numpy as np
import matplotlib.pyplot as plt
from scipy import interpolate
from sklearn import linear_model
import time

def interpolate_DAS_data(data_time_slice, coord_arr, coord_arr_reg):
    x = coord_arr[:,0] 
    y = coord_arr[:,1] 
    f = interpolate.interp1d(x, data_time_slice)
    data_time_slice_interpolate = f(coord_arr_reg[:, 0])
    return data_time_slice_interpolate

def fit_linear_coord(coord_arr, channel_spacing=4.):
    coord_X = coord_arr[:,0]
    coord_Y = coord_arr[:,1]
    coord_X = coord_X.reshape(coord_X.size, 1)
    coord_Y = coord_Y.reshape(coord_Y.size, 1)
    # Create linear regression object
    regr = linear_model.LinearRegression()
    # Train the model using the training sets
    regr.fit(coord_X, coord_Y)
    # Make predictions using the testing set
    coord_X_spacing = channel_spacing * 1. / np.sqrt(regr.coef_**2+1)
    coord_X_reg = np.arange(coord_X[0], coord_X[-1], coord_X_spacing)
    coord_X_reg = coord_X_reg.reshape(coord_X_reg.size, 1)
    coord_Y_reg = regr.predict(coord_X_reg)
    coord_arr_reg = np.concatenate((coord_X_reg, coord_Y_reg), axis=1)
    #coord_arr_reg = coord_arr_reg[:coord_arr.shape[0], :]
    return coord_arr_reg
        

def compute_coord_distance(coord_arr, show=False):
    dist_list = [np.sqrt(np.sum((coord_arr[(j+1),:]-coord_arr[j,:])**2)) for j in range(coord_arr.shape[0]-1)]
    if show:
        plt.hist(dist_list)
        plt.show()
    return dist_list

def extract_Line_data(start_time, wlen, lines, chlist_file_root="/home/syyuan/project/BFM_opt/BFM_new/Txt/",\
                      n_channel=620, plot=True, channel_spacing=4., interpolate=False, reverse_blue_line_od=True):
    chIdx_list = []
    coord_list = []
    for line in lines:
        if line[0] == 'r':
            chlist_file_name = "chlistBeamforming_dir74.txt"
        if line[0] == 'b':
            chlist_file_name = "chlistBeamforming_dir168.txt"
        chlist_file_path = os.path.join(chlist_file_root, chlist_file_name)
        start_ch = line_ch_dict[line+'_st']
        end_ch = line_ch_dict[line+'_ed']
        start_ch_idx = start_ch-1
        end_ch_idx = end_ch-1
        chIdxIn, coordIn, count_ch = extract_coords_from_channel_file(chlist_file_path)
        line_chIdxIn = list(chIdxIn[start_ch_idx:end_ch_idx])
        line_coord_list = np.array(coordIn[start_ch_idx:end_ch_idx, :])
        if reverse_blue_line_od and line[0] == 'b':
            line_chIdxIn = line_chIdxIn[::-1]
            line_coord_list = line_coord_list[::-1,:]
        line_coord_list = list(line_coord_list)
        chIdx_list = chIdx_list+line_chIdxIn
        coord_list = coord_list+line_coord_list
    chIdx_arr = np.array(chIdx_list)
    coord_arr = np.array(coord_list)
    #compute_coord_distance(coord_arr, show=True)
    data_matrix = getDASStrainRate(start_time, wlen, range(n_channel))
    data_matrix = reorder_data(data_matrix, chIdx_arr)
    if interpolate:
        coord_arr_reg = fit_linear_coord(coord_arr, channel_spacing=channel_spacing)
        data_matrix_reg = []
        for it, data_time_slice in enumerate(data_matrix.T):
            data_matrix_reg.append(interpolate_DAS_data(data_time_slice, coord_arr, coord_arr_reg))
        data_matrix_reg = np.array(data_matrix_reg)
        data_matrix_reg = data_matrix_reg.T
        data_matrix = data_matrix_reg
    if plot:
        plot_SDASA_data(data_matrix)
    n_ch = data_matrix.shape[0]
    x_axis = np.arange(0, channel_spacing*n_ch ,channel_spacing)
    return data_matrix, x_axis

def reorder_fk_spec(fk):
    n_s, n_t = fk.shape
    len_freq = n_t 
    fk = np.concatenate((fk[:,(len_freq/2+1):], fk[:,:(len_freq/2+1)]), axis=1)
    #fk = np.concatenate((fk[(len(wavenum)/2):,:], fk[:(len(wavenum)/2), :]), axis=0)
    return fk

def taper_data_matrix(data_matrix, sampling_rate=50, taper_sec=0.2):        
    taper_n_sample = int(taper_sec*sampling_rate)
    taper = np.hanning(taper_n_sample)
    data_matrix[:, :(taper_n_sample/2)] = data_matrix[:, :(taper_n_sample/2)]*taper[:(taper_n_sample/2)]
    data_matrix[:, (-taper_n_sample/2):] = data_matrix[:, (-taper_n_sample/2):]*taper[(taper_n_sample/2):]
    return data_matrix

def FK_transform(data_matrix, sampling_rate=50, spacing=4, taper_sec=0.2):
    n_s, n_t = data_matrix.shape
    data_matrix = taper_data_matrix(data_matrix, sampling_rate, taper_sec)
    fk = np.fft.fft2(data_matrix)
    nyquist_freq = sampling_rate *0.5 
    nyquist_wvnb = 1./spacing*0.5
    fk_half = fk[:(n_s/2), :]*2
    freq = np.fft.fftfreq(n_t, d=1./sampling_rate)
    wavenum = np.fft.fftfreq(n_s, d=spacing)[:(n_s/2)]
    return fk_half, freq, wavenum

    
def twoD_bf_func(iwv, ifreq, wavenum, freq, phase_mat, amplitude_mat, eps, bf_results, slo_axis, d_slo, it_tau_axis, n_t):
    wv = wavenum[iwv]
    f = freq[ifreq]
    slo = -wv/(f+eps)
    islo = (np.round((slo-slo_axis[0])/d_slo)).astype(int)
    phase = phase_mat[iwv, ifreq]
    amplitude = amplitude_mat[iwv, ifreq]
    slo_idx_bool = np.logical_and(islo >=0, islo < slo_axis.size)
    islo = islo[slo_idx_bool]
    amp = amplitude*np.cos(2*np.pi*ifreq*1.*it_tau_axis/n_t+phase)
    bf_results[islo,:] = bf_results[islo,:]+amplitude*np.cos(2*np.pi*ifreq*1.*it_tau_axis/n_t+phase)
    return bf_results

def fast_BF(fk, freq, wavenum, x_axis=None, t_axis=None, eps=1e-10, slo_axis=None, tau_axis=None):
    n_x = x_axis.size
    n_t = t_axis.size
    d_t = t_axis[1] - t_axis[0]
    d_slo = slo_axis[1] - slo_axis[0]
    n_slo = len(slo_axis)
    n_tau = len(tau_axis)
    d_tau = tau_axis[1] - tau_axis[0]
    bf_results = np.zeros((n_slo, n_tau))
    ix_axis = np.arange(x_axis.size)
    it_axis = np.arange(t_axis.size)
    simulated_data = np.zeros((n_x, n_t))  
    phase_mat =  np.angle(fk)
    amplitude_mat = np.absolute(fk)
    k_vec = 2*np.pi*wavenum
    w_vec = 2*np.pi*freq
    it_tau_axis = (np.round((tau_axis-t_axis[0])/d_t)).astype(int)
    iwv_axis = np.arange(wavenum.size)
    ifreq_axis = np.arange(freq.size)
    #bf_results = twoD_bf_func(iwv_axis[:, None], ifreq_axis[None, :], wavenum, freq, phase_mat, amplitude_mat, 1e-10, bf_results, slo_axis, d_slo, it_tau_axis, n_t)
    for iwv, wv in enumerate(wavenum):
        for ifreq, f in enumerate(freq):
            slo = -wv/(f+eps)
            islo = int(round((slo-slo_axis[0])/d_slo))
            if islo >= 0 and islo < n_slo:
                phase = phase_mat[iwv, ifreq]
                amplitude = amplitude_mat[iwv, ifreq]
                bf_results[islo,:] = bf_results[islo,:]+amplitude*np.cos(2*np.pi*ifreq*1.*it_tau_axis/n_t+phase)
    return bf_results

def func(ix, it, phase, ik, nx, iw, nt, amplitude):
    return amplitude*np.cos(2*np.pi*(ik*1.*ix/nx+iw*1.*it/nt)+phase)

def fast_BF_v2(fk, freq, wavenum, x_axis=None, t_axis=None, eps=1e-10, slo_axis=None, tau_axis=None):
    if tau_axis is None:
        tau_axis = t_axis
    n_x = x_axis.size
    n_t = t_axis.size
    n_slo = slo_axis.size
    n_tau = tau_axis.size
    dtau = tau_axis[1]-tau_axis[0]
    ix_axis = np.arange(x_axis.size)
    it_axis = np.arange(t_axis.size)
    d_t = t_axis[1] - t_axis[0]
    t_axis_new = np.arange(t_axis[0], t_axis[-1], dtau)
    it_axis_new = (np.round((tau_axis-t_axis_new[0])/dtau)).astype(int)
    nt_new = it_axis_new.size
    bf_results = np.zeros((n_slo, n_tau))
    simulated_data_fk = np.zeros((n_x, n_t))  
    phase_mat =  np.angle(fk)
    amplitude_mat = np.absolute(fk)
    k_vec = 2*np.pi*wavenum
    w_vec = 2*np.pi*freq
    min_slo = slo_axis[0]
    max_slo = slo_axis[-1]
    d_slo = slo_axis[1] -slo_axis[0]
    for iwv, wv in enumerate(wavenum):
        for ifreq, f in enumerate(freq):
            slo = -wv/(f+1e-10)
            if slo < min_slo or slo > max_slo:
                continue
            islo = int(round((slo-min_slo)/d_slo))
            phase = phase_mat[iwv, ifreq]
            amplitude = amplitude_mat[iwv, ifreq]
            #bf_results[islo, :] = bf_results[islo, :]+amplitude*np.cos(2*np.pi*(ifreq*1.*it_axis_new/n_t)+phase)
            title1 = "fk simulated ... wv: {} f: {}".format(wv, f)
            title2 = "bf simulated ... wv: {} f: {}".format(wv, f)
    #        if iwv >= 2 and ifreq>0:
            bf_results[islo, :] = bf_results[islo, :]+amplitude*np.cos(2*np.pi*(ifreq*1.*it_axis_new/nt_new)+phase)
     #           simulated_data_fk = simulated_data_fk+func(ix_axis[:, None], it_axis[None, :], phase, iwv, n_x, ifreq, n_t, amplitude)
           # simulated_data_bf = bf_simulated_data(bf_results, x_axis=x_axis, t_axis=t_axis_new, slo_axis=slo_axis, tau_axis=tau_axis)
     #           plot_SDASA_data(simulated_data_fk.real, title=title1)
            #plot_SDASA_data(simulated_data_bf.real, title=title2, sampling_rate=1./dtau)
    return bf_results 

def FK_simulated_data(fk, freq, wavenum, x_axis=None, t_axis=None, eps=1e-10, slo_axis=None, tau_axis=None, \
                      sin_wavenum=None, sin_freq=None):
    if tau_axis is None:
        tau_axis = t_axis
    n_x = x_axis.size
    n_t = t_axis.size
    n_slo = slo_axis.size
    n_tau = tau_axis.size
    ix_axis = np.arange(x_axis.size)
    it_axis = np.arange(t_axis.size)
    dtau = tau_axis[1]-tau_axis[0]
    t_axis_new = np.arange(t_axis[0], t_axis[-1]+dtau, dtau)
    it_axis_new = (np.round((tau_axis-t_axis_new[0])/dtau)).astype(int)
    nt_new = it_axis_new.size
    simulated_data = np.zeros((n_x, n_t))  
    phase_mat =  np.angle(fk)
    amplitude_mat = np.absolute(fk)
    k_vec = 2*np.pi*wavenum
    w_vec = 2*np.pi*freq
    #v_vec = freq/(wavenum+eps)
    min_slo = 0
    max_slo = 1e10
    bf_results = np.zeros((n_slo, n_tau))
    if slo_axis is not None:
        min_slo = slo_axis[0]
        max_slo = slo_axis[-1]
        d_slo = slo_axis[1] -slo_axis[0]
    for iwv, wv in enumerate(wavenum):
        for ifreq, f in enumerate(freq):
            phase = phase_mat[iwv, ifreq]
            amplitude = amplitude_mat[iwv, ifreq]
            if sin_wavenum is not None and sin_freq is not None: 
                #if abs(wv - sin_wavenum)<0.003 or abs(f-sin_freq) < 0.003:
                if abs(wv - sin_wavenum)<1e-5 and abs(f-sin_freq) < 1e-5:
                #if abs(f-sin_freq) < 0.003:
                    #print wv, sin_wavenum, abs(wv - sin_wavenum)
                    slo = -wv/(f+1e-10)
                    islo = int(round((slo-min_slo)/d_slo))
                    simulated_data = simulated_data+func(ix_axis[:, None], it_axis[None, :], phase, iwv, n_x, ifreq, n_t, amplitude) 
                    bf_results[islo, :] = bf_results[islo, :]+amplitude*np.cos(2*np.pi*(ifreq*1.*it_axis_new/nt_new)+phase)
            else:
                simulated_data = simulated_data+func(ix_axis[:, None], it_axis[None, :], phase, iwv, n_x, ifreq, n_t, amplitude) 
    if sin_wavenum is not None and sin_freq is not None:
        simulated_data_bf = bf_simulated_data(bf_results, x_axis=x_axis, t_axis=t_axis_new, slo_axis=slo_axis, tau_axis=tau_axis)
        simulated_data_bf = simulated_data_bf[:,:25]
        plot_SDASA_data(simulated_data_bf.real, sampling_rate=1./dtau)
         
    return simulated_data[:,:25] 

def plot_bf_results(bf_results, tau_axis, slo_axis,pclip=98, show_abs=True):
    if show_abs:
        bf_results = abs(bf_results)
    bf_clipval = np.percentile(np.absolute(bf_results),pclip)
    plt.imshow(bf_results, cmap='gray',aspect='auto',interpolation='nearest',extent=[tau_axis[0],tau_axis[-1],slo_axis[-1],slo_axis[0]],vmin=-bf_clipval,vmax=bf_clipval)
    plt.xlabel("tau [s]")
    plt.ylabel("slowness [s/m]")
    plt.show()

def bf_simulated_data(bf_results, x_axis, t_axis, slo_axis, tau_axis, tau_min=None, tau_max=None):
    if tau_min is None:
        tau_min = tau_axis[0]
    if tau_max is None:
        tau_max = tau_axis[-1]
    n_x = x_axis.size
    n_t = t_axis.size
    simulated_data = np.zeros((n_x, n_t))  
    d_t = t_axis[1]-t_axis[0]
    d_x = x_axis[1]-x_axis[0]
    d_tau = tau_axis[1]-tau_axis[0]
    tau_min_idx = int(round((tau_min-tau_axis[0])/d_tau))
    tau_max_idx = int(round((tau_max-tau_axis[0])/d_tau))
    for islo, slo in enumerate(slo_axis):
        for itau, tau in enumerate(tau_axis):
            t = x_axis*slo+tau
            t_idx = (np.round((t-t_axis[0])/d_t)).astype(int)
            t_idx_bool = np.logical_and(t_idx >=0, t_idx < len(t_axis))
            t_idx = t_idx[t_idx_bool]
            x = x_axis[t_idx_bool]
            x_idx = (np.round((x-x_axis[0])/d_x)).astype(int)
            #if bf_results[islo, itau] != 0 and slo > -0.002:
            if itau > tau_min_idx and itau < tau_max_idx:
                simulated_data[x_idx, t_idx] = simulated_data[x_idx, t_idx] + bf_results[islo, itau] 
           # if np.sum(simulated_data!=0) < 100:
           #     continue
           # print tau, slo, bf_results[islo, itau]
           # print np.sum(simulated_data!=0)
           # plot_SDASA_data(simulated_data)
    return simulated_data 

def simulated_periodic_wave(fk, freq, wavenum, x_axis, t_axis, sin_wavenum, sin_freq, slo_axis, tau_axis):
    periodic_data = FK_simulated_data(fk, freq, wavenum, x_axis=x_axis, t_axis=t_axis, \
                      sin_freq=sin_freq, sin_wavenum=sin_wavenum, slo_axis=slo_axis, tau_axis=tau_axis)
    plot_SDASA_data(periodic_data.real)
    
    

def bf_line_data(start_time, wlen, line, sampling_rate, otau, dtau, ntau, \
                oslo, dslo, nslo, tau_min=None, tau_max=None, show_data=False, show_fk=False, show_bf=False, \
                show_bf_abs=False, show_recovered_fk=False, show_recovered_data=False, simualted_periodic=False, \
                sin_wavenum=0.0106382978723, sin_freq=-12):
    max_slo = oslo+nslo*dslo
    slo_axis = np.arange(oslo, max_slo+dslo, dslo)
    dt = 1./sampling_rate
    t_axis = np.arange(0, wlen, dt)
    if dtau is None:
        dtau = dt
    if ntau is None:
        ntau = int(round(len(t_axis)*dt/dtau))
    max_tau = otau+(ntau-1)*dtau
    tau_axis = np.arange(otau, max_tau+dtau, dtau)
    line_data, x_axis = extract_Line_data(start_time, wlen, [line], plot=show_data)
    fk, freq, wavenum = FK_transform(line_data)
    if show_fk:
        fk_clipval = np.percentile(np.absolute(fk),98) 
        fk_show = abs(reorder_fk_spec(fk))
        print fk_show.shape
        plt.imshow(fk_show,cmap='gray',aspect='auto',interpolation='nearest',extent=[freq[len(freq)/2+1],freq[len(freq)/2],wavenum[-1],wavenum[0]],vmin=-fk_clipval,vmax=fk_clipval)
        plt.xlabel('Frequency')
        plt.ylabel('Wavenumber')
        plt.show()
    if simualted_periodic:
        simulated_periodic_wave(fk, freq, wavenum, x_axis, t_axis, sin_wavenum, sin_freq, slo_axis=slo_axis, tau_axis=tau_axis)
    bf_results = fast_BF(fk, freq, wavenum, x_axis=x_axis, t_axis=t_axis, eps=1e-10, slo_axis=slo_axis, tau_axis=tau_axis)
    if show_bf:
        plot_bf_results(bf_results, tau_axis, slo_axis, pclip=99, show_abs=show_bf_abs)
    t_axis_new = np.arange(t_axis[0], t_axis[-1]+dtau, dtau)
    if tau_min is None:
        tau_min = tau_axis[0]
    if tau_max is None:
        tau_max = tau_axis[-1]
    if show_recovered_fk:
        simulated_data = bf_simulated_data(bf_results, x_axis=x_axis, t_axis=t_axis_new, slo_axis=slo_axis, tau_axis=tau_axis, tau_min=tau_min, tau_max=tau_max)
        max_freq = max(freq) 
        fk, freq, wavenum = FK_transform(simulated_data, sampling_rate=1./dtau, spacing=4, taper_sec=0.2)
        n_s, n_t = simulated_data.shape
        max_freq_idx = int(max_freq/max(freq)*n_t/2+1)
        fk = fk[:(n_s/2), :]
        fk = np.concatenate((fk[:,(-max_freq_idx+1):],fk[:,:max_freq_idx]), axis=1)
        fk_clipval = np.percentile(np.absolute(fk),98) 
        plt.imshow(abs(fk),cmap='gray',aspect='auto',interpolation='nearest',extent=[freq[len(freq)/2+1],freq[len(freq)/2],wavenum[-1],wavenum[0]],vmin=-fk_clipval,vmax=fk_clipval)
        plt.xlabel('Frequency')
        plt.ylabel('Wavenumber')
        plt.show()
        if show_recovered_data:
            plot_SDASA_data(simulated_data.real)
            return bf_results
    if show_recovered_data:
        simulated_data = bf_simulated_data(bf_results, x_axis=x_axis, t_axis=t_axis_new, slo_axis=slo_axis, tau_axis=tau_axis, tau_min=tau_min, tau_max=tau_max)
        plot_SDASA_data(simulated_data.real)
        return bf_results
    return bf_results

if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument('-st', '--start_time', default="2017-8-10T03:19:50.120")
    parser.add_argument('-w', '--wlen', type=float, default=0.5)
    parser.add_argument('-ln', '--line', default='r2')
    parser.add_argument('-r', '--sampling_rate', type=float, default=50)
    parser.add_argument('-sp', '--channel_spacing', type=float, default=4)
    parser.add_argument('-otau', '--otau', type=float, default=0)
    parser.add_argument('-dtau', '--dtau', type=float, default=None)
    parser.add_argument('-ntau', '--ntau', type=float, default=None)
    parser.add_argument('-oslo', '--oslo', type=float, default=-0.001)
    parser.add_argument('-dslo', '--dslo', type=float, default=5e-06)
    parser.add_argument('-nslo', '--nslo', type=float, default=400)
    parser.add_argument('-tau_min', '--tau_min', type=float, default=None)
    parser.add_argument('-tau_max', '--tau_max', type=float, default=None)
    parser.add_argument('-sd', '--show_data', default="False")
    parser.add_argument('-sfk', '--show_fk', default="False")
    parser.add_argument('-sbf', '--show_bf', default="False")
    parser.add_argument('-sa', '--show_abs', default="True")
    parser.add_argument('-srd', '--show_recovered_data', default="False")
    parser.add_argument('-srfk', '--show_recovered_fk', default="False")
    parser.add_argument('-spd', '--simualted_periodic', default="False")
    args = parser.parse_args()
    show_data = False
    show_fk = False
    show_bf = False
    show_abs = True
    show_recovered_data = False
    show_recovered_fk = False
    simualted_periodic = False
    if args.show_data == "True":
        show_data = True
    if args.show_fk == "True":
        show_fk = True
    if args.show_bf == "True":
        show_bf = True
    if args.show_abs == "False":
        show_abs = False
    if args.show_recovered_data == "True":
        show_recovered_data = True
    if args.show_recovered_fk == "True":
        show_recovered_fk = True
    if args.simualted_periodic == "True":
        simualted_periodic = True
    start_time = UTCDateTime(args.start_time)
    bf_line_data(start_time, wlen=args.wlen, line=args.line, sampling_rate=args.sampling_rate, \
                otau=args.otau, dtau=args.dtau, ntau=args.ntau, oslo=args.oslo, \
                dslo=args.dslo, nslo=args.nslo, tau_min=args.tau_min, tau_max=args.tau_max, \
                 show_data=show_data, show_fk=show_fk, show_bf=show_bf,\
                show_bf_abs=show_abs, show_recovered_fk=show_recovered_fk, \
                show_recovered_data=show_recovered_data, simualted_periodic=simualted_periodic)

    '''
    line_data, x_axis = extract_Line_data(start_time, args.wlen, [args.line],interpolate=interpolate)
    dt = 1./args.sampling_rate
    t_axis = np.arange(0, args.wlen, dt)
    fk, freq, wavenum = FK_transform(line_data)
    fk_clipval = np.percentile(np.absolute(fk),98) 
    plt.imshow(abs(reorder_fk_spec(fk)),cmap='gray',aspect='auto',interpolation='nearest',extent=[freq[len(freq)/2],freq[len(freq)/2-1],wavenum[-1],wavenum[0]],vmin=-fk_clipval,vmax=fk_clipval)
    plt.xlabel('Frequency')
    plt.ylabel('Wavenumber')
    plt.show()
    dtau = dt
    if args.dtau is not None:
        dtau = args.dtau
    ntau = int(round(len(t_axis)*dt/dtau))
    if args.ntau is not None:
        ntau = args.ntau
    max_tau = args.otau+(ntau-1)*dtau
    tau_axis = np.arange(args.otau, max_tau+dtau, dtau)
    max_slo = args.oslo+args.nslo*args.dslo
    slo_axis = np.arange(args.oslo, max_slo+args.dslo, args.dslo)
    #bf_results = fast_BF(fk, freq, wavenum, x_axis=x_axis, t_axis=t_axis, eps=1e-10, slo_axis=slo_axis, tau_axis=tau_axis)
    #bf_results = fast_BF_v2(fk, freq, wavenum, x_axis=x_axis, t_axis=t_axis, eps=1e-10, slo_axis=slo_axis, tau_axis=tau_axis)
    bf_results = fast_BF(fk, freq, wavenum, x_axis=x_axis, t_axis=t_axis, eps=1e-10, slo_axis=slo_axis, tau_axis=tau_axis)
    plot_bf_results(bf_results, tau_axis, slo_axis, pclip=99.9, show_abs=show_abs)
    t_axis_new = np.arange(t_axis[0], t_axis[-1], dtau)
    tau_min = tau_axis[0]
    if args.tau_min is not None:
        tau_min = args.tau_min
    tau_max = tau_axis[-1]
    if args.tau_max is not None:
        tau_max = args.tau_max
    simulated_data = bf_simulated_data(bf_results, x_axis=x_axis, t_axis=t_axis_new, slo_axis=slo_axis, tau_axis=tau_axis, tau_min=tau_min, tau_max=tau_max)
    #simulated_data = FK_simulated_data(fk, freq, wavenum, x_axis=x_axis, t_axis=t_axis, min_slo=slo_axis[0], max_slo=slo_axis[-1])
    #simulated_data = taper_data_matrix(simulated_data, sampling_rate=args.sampling_rate, taper_sec=0.2)
    fk, freq, wavenum = FK_transform(simulated_data, sampling_rate=1./dtau, spacing=4, taper_sec=0.2)
    n_s, n_t = simulated_data.shape
    nyquist = args.sampling_rate/2
    nyquist_idx = int(nyquist/max(freq)*n_t/2)
    fk = fk[:(n_s/2), :]
    fk = np.concatenate((fk[:,-nyquist_idx:],fk[:,:nyquist_idx]), axis=1)
    fk_clipval = np.percentile(np.absolute(fk),98) 
    plt.imshow(abs(fk),cmap='gray',aspect='auto',interpolation='nearest',extent=[freq[len(freq)/2],freq[len(freq)/2-1],wavenum[-1],wavenum[0]],vmin=-fk_clipval,vmax=fk_clipval)
    plt.xlabel('Frequency')
    plt.ylabel('Wavenumber')
    plt.show()
    #simulated_data = FK_simulated_data(fk, freq, wavenum, x_axis=x_axis, t_axis=t_axis)
    plot_SDASA_data(simulated_data.real)
'''
