import numpy as np
import matplotlib.pyplot as plt
import obspy
import sys
from obspy.clients.fdsn import Client

''' input: 
    channelsWanted: 0-LongPeriod 1-Broadband'''
def getJRSCData(startTimeStr, endTimeStr, channelsWanted, JRchannel, eventName=None, mag=None, plotsWanted=False, sepFileWanted=False, sepFileDatapath='/data/biondo/syyuan/', want_bp=False, flo=0, fhi=25, Nt="BK", station_code = "JRSC", de_mean=False, print_max=False, want_pdf=False, pdf_path=""):
     
    startTime = obspy.UTCDateTime(startTimeStr)
    endTime = obspy.UTCDateTime(endTimeStr)

    client = Client("NCEDC")
    # some steps to explore what data is available at Jasper Ridge if you're interested
    inv = client.get_stations(starttime=startTime , endtime=endTime , network=Nt, station=station_code, location="*", channel=JRchannel, level="response")
    station_coordinates=[]
    channel_coordinates=[]
    for network in inv:
        for station in network:
            station_coordinates.append((network.code, station.code, station.latitude, station.longitude, station.elevation))
            for channel in station:
                print channel.code
                location =  channel.location_code
                channel_coordinates.append((network.code, station.code, channel.location_code, channel.code, channel.latitude, channel.longitude, channel.elevation, channel.depth))


    if channelsWanted == 0:
        # actually get the long period data (better for distant earthquakes)
        st = client.get_waveforms(Nt,station_code,location,'LH*',startTime,endTime, attach_response=True)
        if de_mean == True:
            for itr, tr in enumerate(st.traces):
                st.traces[itr].data = tr.data - np.mean(tr.data)
                print itr

    if channelsWanted == 1:
        # can also get the broadband data (better for local earthquakes)
        st = client.get_waveforms(Nt,station_code,location,JRchannel,startTime,endTime, attach_response=True)
        sampling_rate = st.traces[0].stats['sampling_rate'] 
        print sampling_rate
        if de_mean == True:
            for itr, tr in enumerate(st.traces):
                st.traces[itr].data = tr.data - np.mean(tr.data)
                if print_max == True:
                    print "max value for trace #{} is {}...".format(itr+1, np.amax(tr.data))
        # plot the broadband data
    if want_bp:
        if not de_mean:
            for itr, tr in enumerate(st.traces):
                st.traces[itr].data = tr.data - np.mean(tr.data)
        st.taper(0.1)
        st.filter("bandpass", freqmin=flo, freqmax=fhi)

    if plotsWanted:
        st.plot()
        plt.show()
    
    if want_pdf:
        st.plot(outfile=pdf_path)

    if sepFileWanted == True:
        for trace in st.traces:
            print trace.stats.channel
            headerfilename = station_code+"_"+trace.stats.channel+"_"+eventName+'.H'
            f = open(sepFileDatapath+headerfilename,'w')
            f.write('in=./'+headerfilename+'@\n')
            n1 = str(len(trace.data))
            d1 = str(trace.stats.delta)
            f.write('n1='+n1+' o1=0.0 d1='+d1+''' label1="time(s)"\n''')
            f.write('n2=1 o2=0.0 d2=1\nn3=1 o3=0.0 d3=1.0\n')
            f.write('''gff="-1" hff="-1"''')
            f.close()
            binfilename = headerfilename + '@'
            binf = open(sepFileDatapath+binfilename,'wb')
            binDat = trace.data
            binDat = binDat.astype(np.float32)
            binDat.byteswap(True)
            newByteArray = bytearray(binDat)
            binf.write(newByteArray)
            binf.close()
        ###### SEGY complains about a header issue ######

        ## write the broadband data as a SEG-Y with 3 channels
        #filename = 'broadbandJRSC'+startTimeStr+'.segy'
        #stBroadband.write(filename, format='SEGY', data_encoding=2, byteorder=sys.byteorder)

        ## test that the broadband data was written properly by reading and plotting data from file
        #segy = obspy.io.segy.segy.read(filename)
        #segy.plot()

        ###### so just write it in the same way as long period ########  
        # Write each channel of the long period data to a file with its information
        

    return st
