import sys
from obspy import UTCDateTime
import obspy
import datetime as dt
import numpy as np
import matplotlib.pyplot as plt
from obspy import UTCDateTime
import argparse
from utils import getDASStrainRate
import sqlite3
from getJRSCData import getJRSCData
from readSEPfile import *
from plot_SDASA_data import plot_SDASA_2broadband
from fk_bf import extract_Line_data

if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument('-st','--start_time',help='start time',default=None)
    parser.add_argument('-w','--wlen',help='window length (s)', type=int, default=10)
    parser.add_argument('-bp','--bp',help='if true, do bandpass', type=bool, default=False)
    parser.add_argument('-flo','--flo',help='low_frequency (s)', type=int, default=0.5)
    parser.add_argument('-fhi','--fhi',help='low_frequency (s)', type=int, default=20)
    parser.add_argument('-fn','--output_file',help='file name to save the data', default=None)
    parser.add_argument('-pl','--plot',help='if true, show the plot', type=bool, default=True)
    parser.add_argument('-chls','--channel_list', nargs='+', type=int, default=range(10,200)+range(230,500))
    parser.add_argument('-clip','--clip_val', type=int, default=90)
    parser.add_argument('-o','--output', default=None)
    parser.add_argument('-sr','--sampling_rate', type=int, default=50)
    parser.add_argument('-fp','--file_path', default=None)
    # JRSC parameters
    parser.add_argument('-Jbp','--Jbp',help='if true, do bandpass', default="False")
    parser.add_argument('-Jflo','--Jflo',help='low_frequency (s)', type=float, default=0.5)
    parser.add_argument('-Jfhi','--Jfhi',help='low_frequency (s)', type=float, default=20)
    parser.add_argument('-ch','--channel', default=None)
    parser.add_argument('-nt', '--Nt', default="BK")
    parser.add_argument('-sc', '--station_code', default="JRSC")
    parser.add_argument('-dm', '--de_mean', default="False")
    parser.add_argument('-pll', '--plot_line', default="False")
    parser.add_argument('-J2bp','--J2bp',help='if true, do bandpass', default="False")
    parser.add_argument('-J2flo','--J2flo',help='low_frequency (s)', type=float, default=0)
    parser.add_argument('-J2fhi','--J2fhi',help='low_frequency (s)', type=float, default=50)
    parser.add_argument('-J2ch','--J2channel', default="HNZ")
    parser.add_argument('-J2nt', '--J2Nt', default="NC")
    parser.add_argument('-J2sc', '--J2station_code', default="JSFB")
    parser.add_argument('-recplt', '--plt_rec', default="False")
    parser.add_argument('-reccrx1', '--bttm_lft_coor_x_1', type=float, default=None)
    parser.add_argument('-reccry1', '--bttm_lft_coor_y_1', type=float, default=None)
    parser.add_argument('-reccrx2', '--bttm_lft_coor_x_2', type=float, default=None)
    parser.add_argument('-reccry2', '--bttm_lft_coor_y_2', type=float, default=None)
    parser.add_argument('-reccrx3', '--bttm_lft_coor_x_3', type=float, default=None)
    parser.add_argument('-reccry3', '--bttm_lft_coor_y_3', type=float, default=None)
    parser.add_argument('-recwdth1', '--rec_width1', type=float, default=None)
    parser.add_argument('-rechght1', '--rec_height1', type=float, default=None)
    parser.add_argument('-recwdth2', '--rec_width2', type=float, default=None)
    parser.add_argument('-rechght2', '--rec_height2', type=float, default=None)
    parser.add_argument('-recwdth3', '--rec_width3', type=float, default=None)
    parser.add_argument('-rechght3', '--rec_height3', type=float, default=None)
    parser.add_argument('-ls', '--label_seg', default="True")
    parser.add_argument('-wcl', '--want_channel_label', default="True")

    args = parser.parse_args()

    ''' get Ladara template data'''
    lines = ["r1", "r2", "r3", "b1", "b2", "b3"]
    Jbp = False
    J2bp = False
    de_mean = False
    plot_line = False
    plt_rec = False
    label_seg = False
    want_channel_label = True
    start_time = UTCDateTime(args.start_time)
    end_time = start_time+args.wlen
    channel = args.channel
    if channel is None:
        if args.station_code == "JRSC":
            channel = "BH*"
        else:
            channel = "*"
    if args.Jbp == "True":
        Jbp = True
    if args.J2bp == "True":
        J2bp = True
    if args.plot_line == "True":
        plot_line = True
    if args.plt_rec == "True":
        plt_rec = True
    if args.de_mean == "True":
        de_mean = True
    if args.label_seg == "True":
        label_seg = True
    if args.want_channel_label == "False":
        want_channel_label = False
    data_line_order,x_axis = extract_Line_data(args.start_time, args.wlen, lines, plot=False, reverse_blue_line_od=False)
    broadband_data_st = getJRSCData(args.start_time, str(end_time), 1, channel, Nt=args.Nt, station_code = args.station_code, de_mean=de_mean, want_bp=Jbp, flo=args.Jflo, fhi=args.Jfhi) 
    broadband_data_tr = broadband_data_st.traces[0]
    J2_broadband_data_st = getJRSCData(args.start_time, str(end_time), 1, args.J2channel, Nt=args.J2Nt, station_code = args.J2station_code, de_mean=de_mean, want_bp=J2bp, flo=args.J2flo, fhi=args.J2fhi) 
    J2_broadband_data_tr = J2_broadband_data_st.traces[0]
    #plt.plot(broadband_data_tr.data)
    #plt.show()
    '''
    if isinstance(data, (bool)):
        print "data is not included in the database..."
        sys.exit()
    '''
    bttm_lft_coor_1 = (args.bttm_lft_coor_x_1, args.bttm_lft_coor_y_1)
    bttm_lft_coor_2 = (args.bttm_lft_coor_x_2, args.bttm_lft_coor_y_2)
    bttm_lft_coor_3 = (args.bttm_lft_coor_x_3, args.bttm_lft_coor_y_3)
    bttm_lft_coor = (bttm_lft_coor_1, bttm_lft_coor_2, bttm_lft_coor_3)
    rec_width = (args.rec_width1, args.rec_width2, args.rec_width3)
    rec_height = (args.rec_height1, args.rec_height2, args.rec_height3)
    if args.plot == True:
        plot_SDASA_2broadband(data_line_order, broadband_data_tr.data,J2_broadband_data_tr.data, \
                            clip_val=args.clip_val,\
                            title=str(args.start_time), sampling_rate=args.sampling_rate,file_path=args.file_path,\
                            want_line=plot_line, plot_rec=plt_rec, bttm_lft_coor=bttm_lft_coor, \
                            rec_width=rec_width, rec_height=rec_height, label_segments=label_seg, \
                            want_channel_label=want_channel_label)
