import numpy as np
import sys
import matplotlib.pyplot as plt
from os import listdir
import argparse
from utils import get_xcorr_results_from_time_range, get_xcorr_results_from_folder
from obspy import UTCDateTime
import datetime
import time
import os
from scipy.signal import hilbert


def plt_xcorr_results_folder(folder, sampling_rate=50, want_fig=False, fig_path=None, nthreads=5):
    xcorr_results, time_array = get_xcorr_results_from_folder(folder, sampling_rate=sampling_rate, nthreads=nthreads)
#    xcorr_results[abs(xcorr_results)>1.2] = 1
    plt.ylim(-0.5,1)
    plt.grid()
    if want_fig != "False":
        plt.savefig(fig_path)
    else:
        plt.show()

def plt_xcorr_results_time_range(start_time, end_time, root, tmplt_folder, sampling_rate, want_fig=False, fig_path=None,nthreads=10,channel=None, want_title=True, want_snr=False):
    xcorr_results, time_array = get_xcorr_results_from_time_range(root, tmplt_folder, start_time, end_time, sampling_rate=sampling_rate,nthreads=nthreads)
    xcorr_results[abs(xcorr_results)>1.2] = 0 
    if want_snr:
        #tic = time.time()
        #signal_env= hilbert(xcorr_results)
        #toc = time.time()
        #print "time to calculate hilbert transform is {}".format(toc-tic)
        noise_level = np.percentile(abs(xcorr_results), 99.5)
        signal_level = np.amax(abs(xcorr_results))
        print "noise level is {}".format(noise_level)
        snr = signal_level/noise_level
        print "SNR is {}".format(snr)
   # plt.figure(figsize=(20,10))
    plt.figure(figsize=(10,3.5))
    plt.plot(time_array, xcorr_results)
    if want_snr:
        plt.plot(time_array, [noise_level]*time_array.size)
    plt.ylim(-0.4,1)
    plt.xlabel('Time Lag')
    plt.ylabel('Cross-correlation Amplitude')
    plt.grid()
    if channel is not None:
        station = root.split('/')[-2]
        if want_title:
            title = "{}: {}".format(station, channel)
            plt.title(title) 
    if want_fig != False:
        plt.savefig(fig_path)
        plt.xticks(fontsize=0.5)
        print "{} saved!".format(fig_path)
    else:
        plt.show()

if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument('-st','--start_time',help='start time',default=None)
    parser.add_argument('-ed','--end_time',help='end time',default=None)
    parser.add_argument('-rt','--root',help='npz data root',default='/data/biondo/syyuan/laderaXcorr/')
    parser.add_argument('-tf','--tmplt_folder',help='template used for the xcorrelation',default='0810_mag18')
    parser.add_argument('-sr','--sampling_rate',help='sampling_rate', type=int, default=50)
    parser.add_argument('-wf','--want_fig',help='save figures or not', default="False")
    parser.add_argument('-fp','--fig_path',help='path to save figures', default=None)
    parser.add_argument('-ch','--channel',help='path to save figures', default=None)
    parser.add_argument('-wt','--want_title',help='title of the figure', default="False")
    parser.add_argument('-wsnr','--want_snr',help='title of the figure', default="False")

    args = parser.parse_args()

    start_time = UTCDateTime(args.start_time)
    end_time = UTCDateTime(args.end_time)

    want_fig = True
    want_title = False
    want_snr = False
    if args.want_fig == "False":
        want_fig = False
    if args.want_title == "True":
        want_title = True
    if args.want_snr == "True":
        want_snr = True
    root = args.root
    if args.channel is not None:
        root = os.path.join(args.root, args.channel)
    plt_xcorr_results_time_range(start_time, end_time, root, args.tmplt_folder, \
                            args.sampling_rate, want_fig, args.fig_path, channel=args.channel, \
                            want_title=want_title, want_snr=want_snr)
