import struct
import numpy as np
import os

def readTrace(infile,nSamples,dataLen,endian,nSamplesToRead):
	'''infile is .H, nSamples is the number of samples per sensor, and traceNumber is the sensor number (start with 1),dataLen is number of bytes per data sample'''
	fin = open(infile, 'rb') # open file for reading binary mode
	startData = 0
	fin.seek(startData)
	thisDataBinary = fin.read(nSamplesToRead*dataLen) # read binary bytes from file
	fin.close()
	thisDataArray = struct.unpack_from(endian+('f')*nSamplesToRead,thisDataBinary) # get data as a tuple of floats
	return np.asarray(thisDataArray,dtype=np.float32)

def readHeader(fileName):
	f = open(fileName,'r')
	n1 = 0
	n2 = 0
	dataFileName = ''
	for text in f:
		if 'in=' in text:
			idx = text.find("in")
			if text[idx+3] != '"':
				dataFileName = text[idx+3:-1]
			else:
				dataFileName = text[idx+4:-2]
		if 'n1=' in text:
			#print(text)
			startIdx = text.find('n1=') + 3
			endIdx = text.find(' ', startIdx)
			n1 = int(text[startIdx:endIdx])
		if 'n2=' in text:
			startIdx = text.find('n2=') + 3
			endIdx = text.find(' ', startIdx)
			n2 = int(text[startIdx:endIdx])
	return dataFileName, n1, n2

def readHeader_time_axis(fileName):
    dataFileName, n1, n2 = readHeader(fileName)
    f = open(fileName,'r')
    for text in f:
        if 'o1=' in text:
            startIdx = text.find('o1=') + 3
            endIdx = text.find(' ', startIdx)
            o1 = float(text[startIdx:endIdx])
        if 'd1=' in text:
            startIdx = text.find('d1=') + 3
            endIdx = text.find(' ', startIdx)
            d1 = float(text[startIdx:endIdx])
    return dataFileName, n1, n2, o1, d1

def read_in_data_from_filepath(file_path, bin_path=None, time_offset=0, wlen=None):
    bp, n1, n2, o1, d1 = readHeader_time_axis(file_path)
    if bin_path is None:
        bin_path = file_path+"@"
    start_idx = int((time_offset-o1)/d1)
    if wlen is not None:
        end_idx = start_idx+int(wlen/d1)
    else:
        end_idx = int(n1-(o1/d1)-1)
    data_array = readTrace(bin_path, n1*n2, 4, '>', n1*n2)
    data_matrix = np.reshape(data_array, (n2, n1))
    data_matrix = data_matrix[:,start_idx:end_idx]
    return data_matrix

def extract_coords_from_channel_file(chlistfile_name):
    chlistfile = open(chlistfile_name)
    count_ch = []
    chlistIn = []
    coordIn = []
    count = 0
    for line in chlistfile:
        if len(line.split()) > 2:
            count = count + 1
            chlistIn.append(int(line.split()[0]))
            coordIn.append(map(float, line.split()[1:]))
        else:
            count_ch.append(count)
            count = 0
    count_ch.append(count)
    count_ch = count_ch[1:]
    chlistIn=np.asarray(chlistIn)
    coordIn=np.asarray(coordIn)
    chIdxIn = chlistIn-1
    return chIdxIn, coordIn, count_ch
    
def reorder_data(data_matrix, chIdxIn, pclip=0.98):
    data_matrix_reorder = []
    for ich,ch in enumerate(chIdxIn):
        data_matrix_reorder.append(data_matrix[ch,:])
    data_matrix_reorder = np.asarray(data_matrix_reorder)

    n2,n1 = data_matrix_reorder.shape
    cmin = -np.percentile(np.absolute(data_matrix_reorder),pclip*100)
    cmax = np.percentile(np.absolute(data_matrix_reorder),pclip*100)
    data_matrix_reorder = np.clip(data_matrix_reorder,cmin,cmax)
    return data_matrix_reorder

def write_sep_file(data, par, output_file):
    with open(output_file,'w') as f:
        f.write('in='+output_file+'@\nn1='+str(par["n1"])+' o1=0.0 d1='+str(par["d1"])+'\n')
        f.write('n2='+str(par["n2"])+' o2=0.0 d2=1\n')
        f.write('n3=1 o3=0.0 d3=1.0\n')
        f.write('gff="-1" hff="-1"\n')
    with open(output_file+'@','wb') as newFile:
        rt = np.reshape(data, par["n1"]*par["n2"])
        rt = rt.astype(np.float32)
        rt.byteswap(True)
        newByteArray = bytearray(rt)
        newFile.write(newByteArray)
        print(output_file+" saved.")
